from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ResponseHeader(_message.Message):
    __slots__ = ("success", "response_code", "message")
    SUCCESS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_CODE_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    success: bool
    response_code: int
    message: str
    def __init__(self, success: bool = ..., response_code: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...

class TestCommsRequest(_message.Message):
    __slots__ = ("message",)
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    message: str
    def __init__(self, message: _Optional[str] = ...) -> None: ...

class TestCommsResponse(_message.Message):
    __slots__ = ("response_header", "response")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    response: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., response: _Optional[str] = ...) -> None: ...

class getDIRequest(_message.Message):
    __slots__ = ("di",)
    DI_FIELD_NUMBER: _ClassVar[int]
    di: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, di: _Optional[_Iterable[int]] = ...) -> None: ...

class getDIResponse(_message.Message):
    __slots__ = ("response_header", "di")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    DI_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    di: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., di: _Optional[_Iterable[bool]] = ...) -> None: ...

class getAIRequest(_message.Message):
    __slots__ = ("ai",)
    AI_FIELD_NUMBER: _ClassVar[int]
    ai: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, ai: _Optional[_Iterable[int]] = ...) -> None: ...

class getAIResponse(_message.Message):
    __slots__ = ("response_header", "ai")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    AI_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    ai: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., ai: _Optional[_Iterable[float]] = ...) -> None: ...

class pulseCounterRequest(_message.Message):
    __slots__ = ("di", "edge")
    DI_FIELD_NUMBER: _ClassVar[int]
    EDGE_FIELD_NUMBER: _ClassVar[int]
    di: int
    edge: str
    def __init__(self, di: _Optional[int] = ..., edge: _Optional[str] = ...) -> None: ...

class pulseCounterResponse(_message.Message):
    __slots__ = ("response_header", "di", "value", "dt_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    DI_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    DT_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    di: int
    value: bool
    dt_secs: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., di: _Optional[int] = ..., value: bool = ..., dt_secs: _Optional[float] = ...) -> None: ...

class getDORequest(_message.Message):
    __slots__ = ("do",)
    DO_FIELD_NUMBER: _ClassVar[int]
    do: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, do: _Optional[_Iterable[int]] = ...) -> None: ...

class getDOResponse(_message.Message):
    __slots__ = ("response_header", "do")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    DO_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    do: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., do: _Optional[_Iterable[bool]] = ...) -> None: ...

class setDORequest(_message.Message):
    __slots__ = ("do", "value")
    DO_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    do: _containers.RepeatedScalarFieldContainer[int]
    value: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, do: _Optional[_Iterable[int]] = ..., value: _Optional[_Iterable[bool]] = ...) -> None: ...

class setDOResponse(_message.Message):
    __slots__ = ("response_header", "do")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    DO_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    do: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., do: _Optional[_Iterable[bool]] = ...) -> None: ...

class scheduleDORequest(_message.Message):
    __slots__ = ("do", "value", "time_secs")
    DO_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    do: _containers.RepeatedScalarFieldContainer[int]
    value: _containers.RepeatedScalarFieldContainer[bool]
    time_secs: float
    def __init__(self, do: _Optional[_Iterable[int]] = ..., value: _Optional[_Iterable[bool]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class scheduleDOResponse(_message.Message):
    __slots__ = ("response_header", "do", "time_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    DO_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    do: _containers.RepeatedScalarFieldContainer[bool]
    time_secs: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., do: _Optional[_Iterable[bool]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class getAORequest(_message.Message):
    __slots__ = ("ao",)
    AO_FIELD_NUMBER: _ClassVar[int]
    ao: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, ao: _Optional[_Iterable[int]] = ...) -> None: ...

class getAOResponse(_message.Message):
    __slots__ = ("response_header", "ao")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    AO_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    ao: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., ao: _Optional[_Iterable[float]] = ...) -> None: ...

class getIoTableRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getIoTableResponse(_message.Message):
    __slots__ = ("response_header", "io_table")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    IO_TABLE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    io_table: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., io_table: _Optional[str] = ...) -> None: ...

class setAORequest(_message.Message):
    __slots__ = ("ao", "value")
    AO_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    ao: _containers.RepeatedScalarFieldContainer[int]
    value: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, ao: _Optional[_Iterable[int]] = ..., value: _Optional[_Iterable[float]] = ...) -> None: ...

class setAOResponse(_message.Message):
    __slots__ = ("response_header", "ao")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    AO_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    ao: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., ao: _Optional[_Iterable[float]] = ...) -> None: ...

class scheduleAORequest(_message.Message):
    __slots__ = ("ao", "value", "time_secs")
    AO_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    ao: _containers.RepeatedScalarFieldContainer[int]
    value: _containers.RepeatedScalarFieldContainer[float]
    time_secs: float
    def __init__(self, ao: _Optional[_Iterable[int]] = ..., value: _Optional[_Iterable[float]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class scheduleAOResponse(_message.Message):
    __slots__ = ("response_header", "ao", "time_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    AO_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    ao: _containers.RepeatedScalarFieldContainer[float]
    time_secs: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., ao: _Optional[_Iterable[float]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class getValueRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[_Iterable[str]] = ...) -> None: ...

class getValueResponse(_message.Message):
    __slots__ = ("response_header", "value")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    value: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., value: _Optional[_Iterable[str]] = ...) -> None: ...

class setValueRequest(_message.Message):
    __slots__ = ("name", "value")
    NAME_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    name: _containers.RepeatedScalarFieldContainer[str]
    value: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, name: _Optional[_Iterable[str]] = ..., value: _Optional[_Iterable[str]] = ...) -> None: ...

class setValueResponse(_message.Message):
    __slots__ = ("response_header", "value")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    value: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., value: _Optional[_Iterable[str]] = ...) -> None: ...

class getSystemStatusRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getSystemStatusResponse(_message.Message):
    __slots__ = ("response_header", "input_voltage", "temperature", "rtc_time", "uptime", "system_info", "scheduled_startup_secs", "scheduled_shutdown_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    INPUT_VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    TEMPERATURE_FIELD_NUMBER: _ClassVar[int]
    RTC_TIME_FIELD_NUMBER: _ClassVar[int]
    UPTIME_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_INFO_FIELD_NUMBER: _ClassVar[int]
    SCHEDULED_STARTUP_SECS_FIELD_NUMBER: _ClassVar[int]
    SCHEDULED_SHUTDOWN_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    input_voltage: float
    temperature: float
    rtc_time: str
    uptime: float
    system_info: str
    scheduled_startup_secs: int
    scheduled_shutdown_secs: int
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., input_voltage: _Optional[float] = ..., temperature: _Optional[float] = ..., rtc_time: _Optional[str] = ..., uptime: _Optional[float] = ..., system_info: _Optional[str] = ..., scheduled_startup_secs: _Optional[int] = ..., scheduled_shutdown_secs: _Optional[int] = ...) -> None: ...

class getShutdownImmunityRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getShutdownImmunityResponse(_message.Message):
    __slots__ = ("response_header", "immunity_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    IMMUNITY_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    immunity_secs: int
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., immunity_secs: _Optional[int] = ...) -> None: ...

class scheduleStartupRequest(_message.Message):
    __slots__ = ("time_secs",)
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    time_secs: float
    def __init__(self, time_secs: _Optional[float] = ...) -> None: ...

class scheduleStartupResponse(_message.Message):
    __slots__ = ("response_header", "time_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    time_secs: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class scheduleShutdownRequest(_message.Message):
    __slots__ = ("time_secs",)
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    time_secs: float
    def __init__(self, time_secs: _Optional[float] = ...) -> None: ...

class scheduleShutdownResponse(_message.Message):
    __slots__ = ("response_header", "time_secs")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    TIME_SECS_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    time_secs: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., time_secs: _Optional[float] = ...) -> None: ...

class rebootRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class rebootResponse(_message.Message):
    __slots__ = ("response_header",)
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ...) -> None: ...

class shutdownRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class shutdownResponse(_message.Message):
    __slots__ = ("response_header",)
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ...) -> None: ...

class getTemperatureRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getTemperatureResponse(_message.Message):
    __slots__ = ("response_header", "temperature")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    TEMPERATURE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    temperature: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., temperature: _Optional[float] = ...) -> None: ...

class getInputVoltageRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getInputVoltageResponse(_message.Message):
    __slots__ = ("response_header", "voltage")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    VOLTAGE_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    voltage: float
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., voltage: _Optional[float] = ...) -> None: ...

class loadFirmwareRequest(_message.Message):
    __slots__ = ("url", "target")
    URL_FIELD_NUMBER: _ClassVar[int]
    TARGET_FIELD_NUMBER: _ClassVar[int]
    url: str
    target: str
    def __init__(self, url: _Optional[str] = ..., target: _Optional[str] = ...) -> None: ...

class loadFirmwareResponse(_message.Message):
    __slots__ = ("response_header",)
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ...) -> None: ...

class loadBootloaderRequest(_message.Message):
    __slots__ = ("url",)
    URL_FIELD_NUMBER: _ClassVar[int]
    url: str
    def __init__(self, url: _Optional[str] = ...) -> None: ...

class loadBootloaderResponse(_message.Message):
    __slots__ = ("response_header",)
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ...) -> None: ...

class getFirmwareVersionRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getFirmwareVersionResponse(_message.Message):
    __slots__ = ("response_header", "version")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    version: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., version: _Optional[str] = ...) -> None: ...

class getLocationRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class getLocationResponse(_message.Message):
    __slots__ = ("response_header", "latitude", "longitude", "altitude_m", "accuracy_m", "speed_mps", "heading_deg", "sat_count", "timestamp")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    LATITUDE_FIELD_NUMBER: _ClassVar[int]
    LONGITUDE_FIELD_NUMBER: _ClassVar[int]
    ALTITUDE_M_FIELD_NUMBER: _ClassVar[int]
    ACCURACY_M_FIELD_NUMBER: _ClassVar[int]
    SPEED_MPS_FIELD_NUMBER: _ClassVar[int]
    HEADING_DEG_FIELD_NUMBER: _ClassVar[int]
    SAT_COUNT_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    latitude: float
    longitude: float
    altitude_m: float
    accuracy_m: float
    speed_mps: float
    heading_deg: float
    sat_count: int
    timestamp: str
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., latitude: _Optional[float] = ..., longitude: _Optional[float] = ..., altitude_m: _Optional[float] = ..., accuracy_m: _Optional[float] = ..., speed_mps: _Optional[float] = ..., heading_deg: _Optional[float] = ..., sat_count: _Optional[int] = ..., timestamp: _Optional[str] = ...) -> None: ...

class syncRtcTimeRequest(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class syncRtcTimeResponse(_message.Message):
    __slots__ = ("response_header",)
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ...) -> None: ...

class EventDetail(_message.Message):
    __slots__ = ("event_id", "event", "pin", "value", "time", "cm4_online")
    EVENT_ID_FIELD_NUMBER: _ClassVar[int]
    EVENT_FIELD_NUMBER: _ClassVar[int]
    PIN_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    TIME_FIELD_NUMBER: _ClassVar[int]
    CM4_ONLINE_FIELD_NUMBER: _ClassVar[int]
    event_id: int
    event: str
    pin: int
    value: str
    time: int
    cm4_online: bool
    def __init__(self, event_id: _Optional[int] = ..., event: _Optional[str] = ..., pin: _Optional[int] = ..., value: _Optional[str] = ..., time: _Optional[int] = ..., cm4_online: bool = ...) -> None: ...

class getEventsRequest(_message.Message):
    __slots__ = ("events_from",)
    EVENTS_FROM_FIELD_NUMBER: _ClassVar[int]
    events_from: int
    def __init__(self, events_from: _Optional[int] = ...) -> None: ...

class getEventsResponse(_message.Message):
    __slots__ = ("response_header", "events", "events_synced")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    EVENTS_SYNCED_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    events: _containers.RepeatedCompositeFieldContainer[EventDetail]
    events_synced: bool
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., events: _Optional[_Iterable[_Union[EventDetail, _Mapping]]] = ..., events_synced: bool = ...) -> None: ...

class getDIEventsRequest(_message.Message):
    __slots__ = ("pin", "rising", "falling", "include_system_events", "events_from")
    PIN_FIELD_NUMBER: _ClassVar[int]
    RISING_FIELD_NUMBER: _ClassVar[int]
    FALLING_FIELD_NUMBER: _ClassVar[int]
    INCLUDE_SYSTEM_EVENTS_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FROM_FIELD_NUMBER: _ClassVar[int]
    pin: int
    rising: bool
    falling: bool
    include_system_events: bool
    events_from: int
    def __init__(self, pin: _Optional[int] = ..., rising: bool = ..., falling: bool = ..., include_system_events: bool = ..., events_from: _Optional[int] = ...) -> None: ...

class getDIEventsResponse(_message.Message):
    __slots__ = ("response_header", "events", "events_synced")
    RESPONSE_HEADER_FIELD_NUMBER: _ClassVar[int]
    EVENTS_FIELD_NUMBER: _ClassVar[int]
    EVENTS_SYNCED_FIELD_NUMBER: _ClassVar[int]
    response_header: ResponseHeader
    events: _containers.RepeatedCompositeFieldContainer[EventDetail]
    events_synced: bool
    def __init__(self, response_header: _Optional[_Union[ResponseHeader, _Mapping]] = ..., events: _Optional[_Iterable[_Union[EventDetail, _Mapping]]] = ..., events_synced: bool = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_stubs/device_agent.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dgrpc_stubs/device_agent.proto\x12\x0c\x64\x65vice_agent\"/\n\rRequestHeader\x12\x13\n\x06\x61pp_id\x18\x01 \x01(\tH\x00\x88\x01\x01\x42\t\n\x07_app_id\"\xae\x01\n\x0eResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x14\n\x0c\x63loud_synced\x18\x02 \x01(\x08\x12\x13\n\x0b\x63loud_ready\x18\x05 \x01(\x08\x12\x1a\n\rresponse_code\x18\x03 \x01(\x05H\x00\x88\x01\x01\x12\x1d\n\x10response_message\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x10\n\x0e_response_codeB\x13\n\x11_response_message\"^\n\x0eMessageDetails\x12\x12\n\nmessage_id\x18\x01 \x01(\t\x12\x14\n\x0c\x63hannel_name\x18\x02 \x01(\t\x12\x0f\n\x07payload\x18\x03 \x01(\t\x12\x11\n\ttimestamp\x18\x04 \x01(\t\"L\n\x0e\x43hannelDetails\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\x12\x16\n\taggregate\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0c\n\n_aggregate\"d\n\x0c\x41gentDetails\x12\x10\n\x08\x61gent_id\x18\x01 \x01(\t\x12\x12\n\nagent_name\x18\x02 \x01(\t\x12.\n\x08\x63hannels\x18\x03 \x03(\x0b\x32\x1c.device_agent.ChannelDetails\"P\n\x10TestCommsRequest\x12+\n\x06header\x18\x02 \x01(\x0b\x32\x1b.device_agent.RequestHeader\x12\x0f\n\x07message\x18\x01 \x01(\t\"\\\n\x11TestCommsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x10\n\x08response\x18\x02 \x01(\t\"X\n\x15MessageDetailsRequest\x12+\n\x06header\x18\x02 \x01(\x0b\x32\x1b.device_agent.RequestHeader\x12\x12\n\nmessage_id\x18\x01 \x01(\t\"\x8f\x01\n\x16MessageDetailsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x32\n\x07message\x18\x02 \x01(\x0b\x32\x1c.device_agent.MessageDetailsH\x00\x88\x01\x01\x42\n\n\x08_message\"Z\n\x15\x43hannelDetailsRequest\x12+\n\x06header\x18\x02 \x01(\x0b\x32\x1b.device_agent.RequestHeader\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\"\x8f\x01\n\x16\x43hannelDetailsResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x32\n\x07\x63hannel\x18\x02 \x01(\x0b\x32\x1c.device_agent.ChannelDetailsH\x00\x88\x01\x01\x42\n\n\x08_channel\"_\n\x1a\x43hannelSubscriptionRequest\x12+\n\x06header\x18\x02 \x01(\x0b\x32\x1b.device_agent.RequestHeader\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\"\xd4\x01\n\x1b\x43hannelSubscriptionResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x32\n\x07\x63hannel\x18\x02 \x01(\x0b\x32\x1c.device_agent.ChannelDetailsH\x00\x88\x01\x01\x12\x32\n\x07message\x18\x03 \x01(\x0b\x32\x1c.device_agent.MessageDetailsH\x01\x88\x01\x01\x42\n\n\x08_channelB\n\n\x08_message\"\xb7\x01\n\x13\x43hannelWriteRequest\x12+\n\x06header\x18\x05 \x01(\x0b\x32\x1b.device_agent.RequestHeader\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\x12\x17\n\x0fmessage_payload\x18\x02 \x01(\t\x12\x15\n\x08save_log\x18\x03 \x01(\x08H\x00\x88\x01\x01\x12\x14\n\x07max_age\x18\x04 \x01(\x05H\x01\x88\x01\x01\x42\x0b\n\t_save_logB\n\n\x08_max_age\"?\n\x10\x44\x65\x62ugInfoRequest\x12+\n\x06header\x18\x01 \x01(\x0b\x32\x1b.device_agent.RequestHeader\"a\n\x14\x43hannelWriteResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x12\n\nmessage_id\x18\x02 \x01(\t\"m\n\x11\x44\x65\x62ugChannelState\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tive\x18\x02 \x01(\x08\x12\x18\n\x10local_updated_at\x18\x03 \x01(\x02\x12\x18\n\x10\x63loud_updated_at\x18\x04 \x01(\x02\"s\n\x13\x44\x65\x62ugPendingMessage\x12\x14\n\x0c\x63hannel_name\x18\x01 \x01(\t\x12\x0f\n\x07\x61pp_key\x18\x02 \x01(\t\x12\x10\n\x08save_log\x18\x03 \x01(\x08\x12\x0f\n\x07payload\x18\x04 \x01(\t\x12\x12\n\npublish_in\x18\x05 \x01(\x05\"\xb5\x02\n\x11\x44\x65\x62ugInfoResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\x18\n\x10\x61\x63tive_callbacks\x18\x02 \x03(\t\x12\x15\n\rwss_callbacks\x18\x03 \x03(\t\x12\x16\n\x0ewss_aggregates\x18\x04 \x03(\t\x12\x19\n\x11wss_subscriptions\x18\x05 \x03(\t\x12\x31\n\x08\x63hannels\x18\x06 \x03(\x0b\x32\x1f.device_agent.DebugChannelState\x12;\n\x10pending_messages\x18\x07 \x03(\x0b\x32!.device_agent.DebugPendingMessage\x12\x15\n\rwss_connected\x18\x08 \x01(\x08\"B\n\x13TempAPITokenRequest\x12+\n\x06header\x18\x01 \x01(\x0b\x32\x1b.device_agent.RequestHeader\"\x83\x01\n\x14TempAPITokenResponse\x12\x35\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1c.device_agent.ResponseHeader\x12\r\n\x05token\x18\x02 \x01(\t\x12\x13\n\x0bvalid_until\x18\x03 \x01(\t\x12\x10\n\x08\x65ndpoint\x18\x04 \x01(\t2\xbc\x04\n\x0b\x64\x65viceAgent\x12N\n\tTestComms\x12\x1e.device_agent.TestCommsRequest\x1a\x1f.device_agent.TestCommsResponse\"\x00\x12`\n\x11GetChannelDetails\x12#.device_agent.ChannelDetailsRequest\x1a$.device_agent.ChannelDetailsResponse\"\x00\x12q\n\x16GetChannelSubscription\x12(.device_agent.ChannelSubscriptionRequest\x1a).device_agent.ChannelSubscriptionResponse\"\x00\x30\x01\x12Y\n\x0eWriteToChannel\x12!.device_agent.ChannelWriteRequest\x1a\".device_agent.ChannelWriteResponse\"\x00\x12Q\n\x0cGetDebugInfo\x12\x1e.device_agent.DebugInfoRequest\x1a\x1f.device_agent.DebugInfoResponse\"\x00\x12Z\n\x0fGetTempAPIToken\x12!.device_agent.TempAPITokenRequest\x1a\".device_agent.TempAPITokenResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_stubs.device_agent_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_REQUESTHEADER']._serialized_start=47
  _globals['_REQUESTHEADER']._serialized_end=94
  _globals['_RESPONSEHEADER']._serialized_start=97
  _globals['_RESPONSEHEADER']._serialized_end=271
  _globals['_MESSAGEDETAILS']._serialized_start=273
  _globals['_MESSAGEDETAILS']._serialized_end=367
  _globals['_CHANNELDETAILS']._serialized_start=369
  _globals['_CHANNELDETAILS']._serialized_end=445
  _globals['_AGENTDETAILS']._serialized_start=447
  _globals['_AGENTDETAILS']._serialized_end=547
  _globals['_TESTCOMMSREQUEST']._serialized_start=549
  _globals['_TESTCOMMSREQUEST']._serialized_end=629
  _globals['_TESTCOMMSRESPONSE']._serialized_start=631
  _globals['_TESTCOMMSRESPONSE']._serialized_end=723
  _globals['_MESSAGEDETAILSREQUEST']._serialized_start=725
  _globals['_MESSAGEDETAILSREQUEST']._serialized_end=813
  _globals['_MESSAGEDETAILSRESPONSE']._serialized_start=816
  _globals['_MESSAGEDETAILSRESPONSE']._serialized_end=959
  _globals['_CHANNELDETAILSREQUEST']._serialized_start=961
  _globals['_CHANNELDETAILSREQUEST']._serialized_end=1051
  _globals['_CHANNELDETAILSRESPONSE']._serialized_start=1054
  _globals['_CHANNELDETAILSRESPONSE']._serialized_end=1197
  _globals['_CHANNELSUBSCRIPTIONREQUEST']._serialized_start=1199
  _globals['_CHANNELSUBSCRIPTIONREQUEST']._serialized_end=1294
  _globals['_CHANNELSUBSCRIPTIONRESPONSE']._serialized_start=1297
  _globals['_CHANNELSUBSCRIPTIONRESPONSE']._serialized_end=1509
  _globals['_CHANNELWRITEREQUEST']._serialized_start=1512
  _globals['_CHANNELWRITEREQUEST']._serialized_end=1695
  _globals['_DEBUGINFOREQUEST']._serialized_start=1697
  _globals['_DEBUGINFOREQUEST']._serialized_end=1760
  _globals['_CHANNELWRITERESPONSE']._serialized_start=1762
  _globals['_CHANNELWRITERESPONSE']._serialized_end=1859
  _globals['_DEBUGCHANNELSTATE']._serialized_start=1861
  _globals['_DEBUGCHANNELSTATE']._serialized_end=1970
  _globals['_DEBUGPENDINGMESSAGE']._serialized_start=1972
  _globals['_DEBUGPENDINGMESSAGE']._serialized_end=2087
  _globals['_DEBUGINFORESPONSE']._serialized_start=2090
  _globals['_DEBUGINFORESPONSE']._serialized_end=2399
  _globals['_TEMPAPITOKENREQUEST']._serialized_start=2401
  _globals['_TEMPAPITOKENREQUEST']._serialized_end=2467
  _globals['_TEMPAPITOKENRESPONSE']._serialized_start=2470
  _globals['_TEMPAPITOKENRESPONSE']._serialized_end=2601
  _globals['_DEVICEAGENT']._serialized_start=2604
  _globals['_DEVICEAGENT']._serialized_end=3176
# @@protoc_insertion_point(module_scope)



from .element import Element
from .submodule import Container


class Camera(Container):
    """Represents a camera element in the UI.

    Parameters
    ----------
    name: str
        The name of the camera.
    display_name: str, optional
        The display name of the camera.
    uri: str, optional
        The URI for the camera feed.
    output_type: str, optional
        The type of output (e.g., 'mp4').
    mp4_output_length: int, optional
        The length of the MP4 output in seconds.
    wake_delay: int, optional
        The delay before the camera wakes up, in seconds.
    children: list[:class:`Element`], optional
        Optional child elements of the camera.
    """

    type = "uiCamera"

    def __init__(
        self,
        name,
        display_name: str = None,
        uri: str = None,
        output_type: str = None,
        mp4_output_length: int = None,
        wake_delay: int = 5,
        children: list[Element] | None = None,
        **kwargs,
    ):
        super().__init__(
            name,
            display_name,
            children=children,
            **kwargs,
            is_available=None,
            help_str=None,
        )
        # fixme: do we need to specify is_available and help_str to be None?

        self.uri = uri
        self.output_type = output_type
        self.mp4_output_length = mp4_output_length
        self.wake_delay = wake_delay

    def to_dict(self):
        # Need to override the to_dict method and ensure that if the children field is an empty dict, it is removed
        result = super().to_dict()
        if not self.children:
            result.pop("children", None)
        return result


class CameraHistory(Camera):
    """Represents a camera history element in the UI."""

    type = "uiCameraFeed"



import enum
import logging

from datetime import datetime
from typing import Optional, Any

from .misc import Colour


class NotSet:
    pass


log = logging.getLogger(__name__)


class Element:
    """Base class for UI elements.

    Attributes
    ----------
    name: str
        The name of the UI element.
    display_name: str, optional
        The display name of the UI element.
    is_available: bool, optional
        Indicates if the UI element is available.
    help_str: str, optional
        A help string for the UI element.
    verbose_str: str, optional
        A verbose string for the UI element.
    show_activity: bool
        Whether to show activity for the UI element.
    form: str, optional
        The form associated with the UI element.
    graphic: str, optional
        The graphic associated with the UI element.
    layout: str, optional
        The layout associated with the UI element.
    component_url: str, optional
        The URL for a remote component associated with the UI element.
    position: int, optional
        The position of the UI element in the UI.
    conditions: dict[str, Any], optional
        Conditions under which the UI element is displayed.
    hidden: bool
        Whether the UI element is hidden.
    """

    type = "uiElement"

    def __init__(
        self,
        name: str,
        optional,
        display_name: str | None = None,
        is_available: bool = None,  # not sure of type
        help_str: str = None,
        verbose_str: str = None,
        show_activity: bool = True,
        form: str = None,
        graphic: str = None,  # not sure of type
        layout: str = None,  # not sure of type
        component_url: str = None,  # not sure of type
        position: Optional[int] = None,  # 100,
        conditions: Optional[dict] = None,
        hidden: bool = False,
        **kwargs,
    ):
        self.name = name
        self.display_name = display_name or kwargs.get(
            "display_str", None
        )  # backwards compatibility
        self.is_available = is_available
        self.help_str = help_str
        self.verbose_str = verbose_str
        self.show_activity = show_activity
        self.form = form
        self.graphic = graphic
        self.layout = layout
        self.component_url = component_url
        self.position = position
        self.conditions = conditions
        self.hidden = hidden

        for k, v in kwargs.items():
            log.debug(f"Setting kwarg {k} to {v}")
            setattr(self, k, v)

        self._retain_fields = []

    def to_dict(self):
        """Convert the element to a dictionary representation.

        Returns
        -------
        dict[str, Any]
            A dictionary representation of the UI element. This is used to serialize the element for the site API.
        """
        to_return = {
            "name": self.name,
            "type": self.type,
            "displayString": self.display_name,
            "isAvailable": self.is_available,
            "helpString": self.help_str,
            "verboseString": self.verbose_str,
            "showActivity": self.show_activity,
            "form": self.form,
            "graphic": self.graphic,
            "layout": self.layout,
            "componentUrl": self.component_url,
            "position": self.position,
            "conditions": self.conditions,
            "hidden": self.hidden,
        }
        # filter out any null values
        return {k: v for k, v in to_return.items() if v is not None}

    def get_diff(
        self,
        other: dict[str, Any],
        remove: bool = True,
        retain_fields: list[str] | None = None,
    ) -> dict[str, Any] | None:
        """Get the difference between this element and another dictionary representation of an element.

        Parameters
        ----------
        other: dict[str, Any]
            The other element to compare against.
        remove: bool
            If True, fields that are not present in this element but are in the other will be set to None.
        retain_fields: list[str], optional
            A list of fields to retain in the difference, even if they are not different.

        Returns
        -------
        dict[str, Any], optional
            A dictionary containing the differences between this element and the other. If there are no differences, returns None.
        """

        this = self.to_dict()

        retain_fields = retain_fields or []
        to_retain = list(set(retain_fields) | set(self._retain_fields))
        # if this == other:
        #     return None

        result = {k: v for k, v in this.items() if other.get(k) != v or k in to_retain}
        if remove:
            result.update(
                **{k: None for k in other if k not in this and k not in to_retain}
            )  # to_remove
        if len(result) == 0:
            return None

        return result

    ## A stub for the method that will be called when the UI state is updated.
    # The element can choose to update its internal state based on the previous state and the new state.
    def recv_ui_state_update(self, state: dict[str, Any]) -> None:
        pass


class ConnectionType(enum.Enum):
    constant = "constant"
    periodic = "periodic"
    other = "other"


class ConnectionInfo(Element):
    """Connection Info

    Parameters
    ----------
    name: str
    connection_type: ConnectionType

    connection_period: int
        The expected time between connection events (seconds) - only applicable for "periodic"
    next_connection: int
        Expected time of next connection (seconds after shutdown)
    offline_after: int
        Show as offline if disconnected for more than x secs

    """

    type = "uiConnectionInfo"

    def __init__(
        self,
        name: str = "connectionInfo",
        connection_type: ConnectionType = ConnectionType.constant,
        connection_period: int = None,
        next_connection: int = None,
        offline_after: int = None,
        allowed_misses: int = None,
        **kwargs,
    ):
        super().__init__(name, None, **kwargs)
        self.name = name
        self.connection_type = connection_type
        self.connection_period = connection_period
        self.next_connection = next_connection
        self.offline_after = offline_after
        self.allowed_misses = allowed_misses

        if self.connection_type is not ConnectionType.periodic and (
            self.connection_period is not None
            or self.next_connection is not None
            or self.allowed_misses is not None
        ):
            raise RuntimeError(
                "connection_type must be periodic to set connection_period, next_connection or offline_after"
            )

    def to_dict(self):
        result = {
            "name": self.name,
            "type": self.type,
            "connectionType": self.connection_type.value,
        }

        if self.connection_period is not None:
            result["connectionPeriod"] = self.connection_period
        if self.next_connection is not None:
            result["nextConnection"] = self.next_connection
        if self.offline_after is not None:
            result["offlineAfter"] = self.offline_after
        if self.allowed_misses is not None:
            result["allowedMisses"] = self.allowed_misses

        return result


class AlertStream(Element):
    """Represents an Alert Stream UI Element

    Parameters
    ----------
    name: str
        The name of the alert stream.
    display_name: str, optional
        The display name of the alert stream.
    """

    type = "uiAlertStream"

    def __init__(self, name, display_name: str = None, **kwargs):
        super().__init__(name, display_name, is_available=None, help_str=None, **kwargs)


class Multiplot(Element):
    """Represents a MultiPlot UI Element.

    Parameters
    ----------
    name: str
        The name of the multiplot.
    display_name: str
        The display name of the multiplot.
    series: list[str]
        A list of series names to be displayed in the multiplot.
    series_colours: list[Colour], optional
        A list of colours for each series in the multiplot.
    series_active: list[bool], optional
        A list indicating whether each series is active or not.
    earliest_data_time: datetime, optional
        The earliest time for which data is available in the multiplot.
    title: str, optional
        The title of the multiplot.
    """

    type = "uiMultiPlot"

    def __init__(
        self,
        name: str,
        display_name: str,
        series: list[str],
        series_colours: Optional[list[Colour]] = None,
        series_active: Optional[list[bool]] = None,
        earliest_data_time: Optional[datetime] = None,
        title: Optional[str] = None,
        shared_axis: Optional[list[bool]] = None,
        step_labels: Optional[list[str]] = None,
        step_padding: Optional[list[int]] = None,
        default_zoom: Optional[str] = None,
        **kwargs,
    ):
        super().__init__(name, display_name, **kwargs)

        self.series = series
        self.series_colours = series_colours
        self.series_active = series_active
        self.earliest_data_time = earliest_data_time
        self.title = title
        self.shared_axis = shared_axis
        self.step_labels = step_labels
        self.step_padding = step_padding
        self.default_zoom = default_zoom

    def to_dict(self):
        result = super().to_dict()
        result["series"] = self.series
        result["colours"] = self.series_colours

        if self.series_active is not None:
            result["activeSeries"] = self.series_active
        if self.shared_axis is not None:
            result["sharedAxis"] = self.shared_axis
        if self.step_labels is not None:
            result["stepLabels"] = self.step_labels
        if self.step_padding is not None:
            result["stepPadding"] = self.step_padding
        if self.default_zoom is not None:
            result["defaultZoom"] = self.default_zoom
        if self.title is not None:
            result["title"] = self.title

        if self.earliest_data_time is not None:
            if isinstance(self.earliest_data_time, datetime):
                result["earliestDataDate"] = int(self.earliest_data_time.timestamp())
            else:
                result["earliestDataDate"] = self.earliest_data_time

        return result


class RemoteComponent(Element):
    """Represents a remote component in the UI.

    Parameters
    ----------
    name: str
        The name of the remote component.
    display_name: str
        The display name of the remote component.
    component_url: str
        The URL of the remote component.
    """

    type = "uiRemoteComponent"

    def __init__(self, name: str, display_name: str, component_url: str, **kwargs):
        super().__init__(name, display_name, component_url=component_url)
        self.kwargs = kwargs

    def to_dict(self):
        res = super().to_dict()
        res.update(self.kwargs)
        return res



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_stubs/platform_iface.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fgrpc_stubs/platform_iface.proto\x12\x0eplatform_iface\"q\n\x0eResponseHeader\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\rresponse_code\x18\x03 \x01(\x05H\x00\x88\x01\x01\x12\x14\n\x07message\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x10\n\x0e_response_codeB\n\n\x08_message\"#\n\x10TestCommsRequest\x12\x0f\n\x07message\x18\x01 \x01(\t\"^\n\x11TestCommsResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x10\n\x08response\x18\x02 \x01(\t\"\x1a\n\x0cgetDIRequest\x12\n\n\x02\x64i\x18\x01 \x03(\x05\"T\n\rgetDIResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x64i\x18\x02 \x03(\x08\"\x1a\n\x0cgetAIRequest\x12\n\n\x02\x61i\x18\x01 \x03(\x05\"T\n\rgetAIResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x61i\x18\x02 \x03(\x02\"/\n\x13pulseCounterRequest\x12\n\n\x02\x64i\x18\x01 \x01(\x05\x12\x0c\n\x04\x65\x64ge\x18\x02 \x01(\t\"\xa7\x01\n\x14pulseCounterResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x0f\n\x02\x64i\x18\x02 \x01(\x05H\x00\x88\x01\x01\x12\x12\n\x05value\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x14\n\x07\x64t_secs\x18\x04 \x01(\x02H\x02\x88\x01\x01\x42\x05\n\x03_diB\x08\n\x06_valueB\n\n\x08_dt_secs\"\x1a\n\x0cgetDORequest\x12\n\n\x02\x64o\x18\x01 \x03(\x05\"T\n\rgetDOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x64o\x18\x02 \x03(\x08\")\n\x0csetDORequest\x12\n\n\x02\x64o\x18\x01 \x03(\x05\x12\r\n\x05value\x18\x02 \x03(\x08\"T\n\rsetDOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x64o\x18\x02 \x03(\x08\"T\n\x11scheduleDORequest\x12\n\n\x02\x64o\x18\x01 \x03(\x05\x12\r\n\x05value\x18\x02 \x03(\x08\x12\x16\n\ttime_secs\x18\x03 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"\x7f\n\x12scheduleDOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x64o\x18\x02 \x03(\x08\x12\x16\n\ttime_secs\x18\x03 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"\x1a\n\x0cgetAORequest\x12\n\n\x02\x61o\x18\x01 \x03(\x05\"T\n\rgetAOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x61o\x18\x02 \x03(\x02\"\x13\n\x11getIoTableRequest\"q\n\x12getIoTableResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x15\n\x08io_table\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_io_table\")\n\x0csetAORequest\x12\n\n\x02\x61o\x18\x01 \x03(\x05\x12\r\n\x05value\x18\x03 \x03(\x02\"T\n\rsetAOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x61o\x18\x02 \x03(\x02\"T\n\x11scheduleAORequest\x12\n\n\x02\x61o\x18\x01 \x03(\x05\x12\r\n\x05value\x18\x02 \x03(\x02\x12\x16\n\ttime_secs\x18\x03 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"\x7f\n\x12scheduleAOResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\n\n\x02\x61o\x18\x02 \x03(\x02\x12\x16\n\ttime_secs\x18\x03 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"\x1f\n\x0fgetValueRequest\x12\x0c\n\x04name\x18\x01 \x03(\t\"Z\n\x10getValueResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\r\n\x05value\x18\x02 \x03(\t\".\n\x0fsetValueRequest\x12\x0c\n\x04name\x18\x01 \x03(\t\x12\r\n\x05value\x18\x02 \x03(\t\"Z\n\x10setValueResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\r\n\x05value\x18\x02 \x03(\t\"\x18\n\x16getSystemStatusRequest\"\x9a\x03\n\x17getSystemStatusResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x1a\n\rinput_voltage\x18\x02 \x01(\x02H\x00\x88\x01\x01\x12\x18\n\x0btemperature\x18\x03 \x01(\x02H\x01\x88\x01\x01\x12\x15\n\x08rtc_time\x18\x04 \x01(\tH\x02\x88\x01\x01\x12\x13\n\x06uptime\x18\x05 \x01(\x02H\x03\x88\x01\x01\x12\x18\n\x0bsystem_info\x18\x06 \x01(\tH\x04\x88\x01\x01\x12#\n\x16scheduled_startup_secs\x18\x07 \x01(\x05H\x05\x88\x01\x01\x12$\n\x17scheduled_shutdown_secs\x18\x08 \x01(\x05H\x06\x88\x01\x01\x42\x10\n\x0e_input_voltageB\x0e\n\x0c_temperatureB\x0b\n\t_rtc_timeB\t\n\x07_uptimeB\x0e\n\x0c_system_infoB\x19\n\x17_scheduled_startup_secsB\x1a\n\x18_scheduled_shutdown_secs\"\x1c\n\x1agetShutdownImmunityRequest\"\x84\x01\n\x1bgetShutdownImmunityResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x1a\n\rimmunity_secs\x18\x02 \x01(\x05H\x00\x88\x01\x01\x42\x10\n\x0e_immunity_secs\">\n\x16scheduleStartupRequest\x12\x16\n\ttime_secs\x18\x01 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"x\n\x17scheduleStartupResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x16\n\ttime_secs\x18\x02 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"?\n\x17scheduleShutdownRequest\x12\x16\n\ttime_secs\x18\x01 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"y\n\x18scheduleShutdownResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x16\n\ttime_secs\x18\x02 \x01(\x02H\x00\x88\x01\x01\x42\x0c\n\n_time_secs\"\x0f\n\rrebootRequest\"I\n\x0erebootResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\"\x11\n\x0fshutdownRequest\"K\n\x10shutdownResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\"\x17\n\x15getTemperatureRequest\"{\n\x16getTemperatureResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x18\n\x0btemperature\x18\x02 \x01(\x02H\x00\x88\x01\x01\x42\x0e\n\x0c_temperature\"\x18\n\x16getInputVoltageRequest\"t\n\x17getInputVoltageResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x14\n\x07voltage\x18\x02 \x01(\x02H\x00\x88\x01\x01\x42\n\n\x08_voltage\"2\n\x13loadFirmwareRequest\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0e\n\x06target\x18\x02 \x01(\t\"O\n\x14loadFirmwareResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\"$\n\x15loadBootloaderRequest\x12\x0b\n\x03url\x18\x01 \x01(\t\"Q\n\x16loadBootloaderResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\"\x1b\n\x19getFirmwareVersionRequest\"f\n\x1agetFirmwareVersionResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x14\n\x12getLocationRequest\"\x84\x03\n\x13getLocationResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12\x15\n\x08latitude\x18\x02 \x01(\x02H\x00\x88\x01\x01\x12\x16\n\tlongitude\x18\x03 \x01(\x02H\x01\x88\x01\x01\x12\x17\n\naltitude_m\x18\x04 \x01(\x02H\x02\x88\x01\x01\x12\x17\n\naccuracy_m\x18\x05 \x01(\x02H\x03\x88\x01\x01\x12\x16\n\tspeed_mps\x18\x06 \x01(\x02H\x04\x88\x01\x01\x12\x18\n\x0bheading_deg\x18\x07 \x01(\x02H\x05\x88\x01\x01\x12\x16\n\tsat_count\x18\x08 \x01(\x05H\x06\x88\x01\x01\x12\x16\n\ttimestamp\x18\t \x01(\tH\x07\x88\x01\x01\x42\x0b\n\t_latitudeB\x0c\n\n_longitudeB\r\n\x0b_altitude_mB\r\n\x0b_accuracy_mB\x0c\n\n_speed_mpsB\x0e\n\x0c_heading_degB\x0c\n\n_sat_countB\x0c\n\n_timestamp\"\x14\n\x12syncRtcTimeRequest\"N\n\x13syncRtcTimeResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\"\x80\x01\n\x0b\x45ventDetail\x12\x10\n\x08\x65vent_id\x18\x01 \x01(\x05\x12\r\n\x05\x65vent\x18\x02 \x01(\t\x12\x0b\n\x03pin\x18\x03 \x01(\x05\x12\r\n\x05value\x18\x04 \x01(\t\x12\x0c\n\x04time\x18\x05 \x01(\x03\x12\x17\n\ncm4_online\x18\x06 \x01(\x08H\x00\x88\x01\x01\x42\r\n\x0b_cm4_online\"<\n\x10getEventsRequest\x12\x18\n\x0b\x65vents_from\x18\x01 \x01(\x03H\x00\x88\x01\x01\x42\x0e\n\x0c_events_from\"\xa7\x01\n\x11getEventsResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12+\n\x06\x65vents\x18\x02 \x03(\x0b\x32\x1b.platform_iface.EventDetail\x12\x1a\n\revents_synced\x18\x03 \x01(\x08H\x00\x88\x01\x01\x42\x10\n\x0e_events_synced\"\x8b\x01\n\x12getDIEventsRequest\x12\x0b\n\x03pin\x18\x01 \x01(\x05\x12\x0e\n\x06rising\x18\x02 \x01(\x08\x12\x0f\n\x07\x66\x61lling\x18\x03 \x01(\x08\x12\x1d\n\x15include_system_events\x18\x04 \x01(\x08\x12\x18\n\x0b\x65vents_from\x18\x05 \x01(\x03H\x00\x88\x01\x01\x42\x0e\n\x0c_events_from\"\xa9\x01\n\x13getDIEventsResponse\x12\x37\n\x0fresponse_header\x18\x01 \x01(\x0b\x32\x1e.platform_iface.ResponseHeader\x12+\n\x06\x65vents\x18\x02 \x03(\x0b\x32\x1b.platform_iface.EventDetail\x12\x1a\n\revents_synced\x18\x03 \x01(\x08H\x00\x88\x01\x01\x42\x10\n\x0e_events_synced2\xbb\x13\n\rplatformIface\x12R\n\tTestComms\x12 .platform_iface.TestCommsRequest\x1a!.platform_iface.TestCommsResponse\"\x00\x12\x46\n\x05getDI\x12\x1c.platform_iface.getDIRequest\x1a\x1d.platform_iface.getDIResponse\"\x00\x12\x46\n\x05getAI\x12\x1c.platform_iface.getAIRequest\x1a\x1d.platform_iface.getAIResponse\"\x00\x12\x62\n\x11startPulseCounter\x12#.platform_iface.pulseCounterRequest\x1a$.platform_iface.pulseCounterResponse\"\x00\x30\x01\x12\x46\n\x05getDO\x12\x1c.platform_iface.getDORequest\x1a\x1d.platform_iface.getDOResponse\"\x00\x12\x46\n\x05setDO\x12\x1c.platform_iface.setDORequest\x1a\x1d.platform_iface.setDOResponse\"\x00\x12U\n\nscheduleDO\x12!.platform_iface.scheduleDORequest\x1a\".platform_iface.scheduleDOResponse\"\x00\x12\x46\n\x05getAO\x12\x1c.platform_iface.getAORequest\x1a\x1d.platform_iface.getAOResponse\"\x00\x12\x46\n\x05setAO\x12\x1c.platform_iface.setAORequest\x1a\x1d.platform_iface.setAOResponse\"\x00\x12U\n\nscheduleAO\x12!.platform_iface.scheduleAORequest\x1a\".platform_iface.scheduleAOResponse\"\x00\x12O\n\x08getValue\x12\x1f.platform_iface.getValueRequest\x1a .platform_iface.getValueResponse\"\x00\x12O\n\x08setValue\x12\x1f.platform_iface.setValueRequest\x1a .platform_iface.setValueResponse\"\x00\x12R\n\tgetEvents\x12 .platform_iface.getEventsRequest\x1a!.platform_iface.getEventsResponse\"\x00\x12X\n\x0bgetDIEvents\x12\".platform_iface.getDIEventsRequest\x1a#.platform_iface.getDIEventsResponse\"\x00\x12\x64\n\x0fgetSystemStatus\x12&.platform_iface.getSystemStatusRequest\x1a\'.platform_iface.getSystemStatusResponse\"\x00\x12\x64\n\x0fgetInputVoltage\x12&.platform_iface.getInputVoltageRequest\x1a\'.platform_iface.getInputVoltageResponse\"\x00\x12\x61\n\x0egetTemperature\x12%.platform_iface.getTemperatureRequest\x1a&.platform_iface.getTemperatureResponse\"\x00\x12U\n\ngetIoTable\x12!.platform_iface.getIoTableRequest\x1a\".platform_iface.getIoTableResponse\"\x00\x12X\n\x0bsyncRtcTime\x12\".platform_iface.syncRtcTimeRequest\x1a#.platform_iface.syncRtcTimeResponse\"\x00\x12X\n\x0bgetLocation\x12\".platform_iface.getLocationRequest\x1a#.platform_iface.getLocationResponse\"\x00\x12p\n\x13getShutdownImmunity\x12*.platform_iface.getShutdownImmunityRequest\x1a+.platform_iface.getShutdownImmunityResponse\"\x00\x12\x64\n\x0fscheduleStartup\x12&.platform_iface.scheduleStartupRequest\x1a\'.platform_iface.scheduleStartupResponse\"\x00\x12g\n\x10scheduleShutdown\x12\'.platform_iface.scheduleShutdownRequest\x1a(.platform_iface.scheduleShutdownResponse\"\x00\x12I\n\x06reboot\x12\x1d.platform_iface.rebootRequest\x1a\x1e.platform_iface.rebootResponse\"\x00\x12O\n\x08shutdown\x12\x1f.platform_iface.shutdownRequest\x1a .platform_iface.shutdownResponse\"\x00\x12[\n\x0cloadFirmware\x12#.platform_iface.loadFirmwareRequest\x1a$.platform_iface.loadFirmwareResponse\"\x00\x12\x61\n\x0eloadBootloader\x12%.platform_iface.loadBootloaderRequest\x1a&.platform_iface.loadBootloaderResponse\"\x00\x12m\n\x12getFirmwareVersion\x12).platform_iface.getFirmwareVersionRequest\x1a*.platform_iface.getFirmwareVersionResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_stubs.platform_iface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_RESPONSEHEADER']._serialized_start=51
  _globals['_RESPONSEHEADER']._serialized_end=164
  _globals['_TESTCOMMSREQUEST']._serialized_start=166
  _globals['_TESTCOMMSREQUEST']._serialized_end=201
  _globals['_TESTCOMMSRESPONSE']._serialized_start=203
  _globals['_TESTCOMMSRESPONSE']._serialized_end=297
  _globals['_GETDIREQUEST']._serialized_start=299
  _globals['_GETDIREQUEST']._serialized_end=325
  _globals['_GETDIRESPONSE']._serialized_start=327
  _globals['_GETDIRESPONSE']._serialized_end=411
  _globals['_GETAIREQUEST']._serialized_start=413
  _globals['_GETAIREQUEST']._serialized_end=439
  _globals['_GETAIRESPONSE']._serialized_start=441
  _globals['_GETAIRESPONSE']._serialized_end=525
  _globals['_PULSECOUNTERREQUEST']._serialized_start=527
  _globals['_PULSECOUNTERREQUEST']._serialized_end=574
  _globals['_PULSECOUNTERRESPONSE']._serialized_start=577
  _globals['_PULSECOUNTERRESPONSE']._serialized_end=744
  _globals['_GETDOREQUEST']._serialized_start=746
  _globals['_GETDOREQUEST']._serialized_end=772
  _globals['_GETDORESPONSE']._serialized_start=774
  _globals['_GETDORESPONSE']._serialized_end=858
  _globals['_SETDOREQUEST']._serialized_start=860
  _globals['_SETDOREQUEST']._serialized_end=901
  _globals['_SETDORESPONSE']._serialized_start=903
  _globals['_SETDORESPONSE']._serialized_end=987
  _globals['_SCHEDULEDOREQUEST']._serialized_start=989
  _globals['_SCHEDULEDOREQUEST']._serialized_end=1073
  _globals['_SCHEDULEDORESPONSE']._serialized_start=1075
  _globals['_SCHEDULEDORESPONSE']._serialized_end=1202
  _globals['_GETAOREQUEST']._serialized_start=1204
  _globals['_GETAOREQUEST']._serialized_end=1230
  _globals['_GETAORESPONSE']._serialized_start=1232
  _globals['_GETAORESPONSE']._serialized_end=1316
  _globals['_GETIOTABLEREQUEST']._serialized_start=1318
  _globals['_GETIOTABLEREQUEST']._serialized_end=1337
  _globals['_GETIOTABLERESPONSE']._serialized_start=1339
  _globals['_GETIOTABLERESPONSE']._serialized_end=1452
  _globals['_SETAOREQUEST']._serialized_start=1454
  _globals['_SETAOREQUEST']._serialized_end=1495
  _globals['_SETAORESPONSE']._serialized_start=1497
  _globals['_SETAORESPONSE']._serialized_end=1581
  _globals['_SCHEDULEAOREQUEST']._serialized_start=1583
  _globals['_SCHEDULEAOREQUEST']._serialized_end=1667
  _globals['_SCHEDULEAORESPONSE']._serialized_start=1669
  _globals['_SCHEDULEAORESPONSE']._serialized_end=1796
  _globals['_GETVALUEREQUEST']._serialized_start=1798
  _globals['_GETVALUEREQUEST']._serialized_end=1829
  _globals['_GETVALUERESPONSE']._serialized_start=1831
  _globals['_GETVALUERESPONSE']._serialized_end=1921
  _globals['_SETVALUEREQUEST']._serialized_start=1923
  _globals['_SETVALUEREQUEST']._serialized_end=1969
  _globals['_SETVALUERESPONSE']._serialized_start=1971
  _globals['_SETVALUERESPONSE']._serialized_end=2061
  _globals['_GETSYSTEMSTATUSREQUEST']._serialized_start=2063
  _globals['_GETSYSTEMSTATUSREQUEST']._serialized_end=2087
  _globals['_GETSYSTEMSTATUSRESPONSE']._serialized_start=2090
  _globals['_GETSYSTEMSTATUSRESPONSE']._serialized_end=2500
  _globals['_GETSHUTDOWNIMMUNITYREQUEST']._serialized_start=2502
  _globals['_GETSHUTDOWNIMMUNITYREQUEST']._serialized_end=2530
  _globals['_GETSHUTDOWNIMMUNITYRESPONSE']._serialized_start=2533
  _globals['_GETSHUTDOWNIMMUNITYRESPONSE']._serialized_end=2665
  _globals['_SCHEDULESTARTUPREQUEST']._serialized_start=2667
  _globals['_SCHEDULESTARTUPREQUEST']._serialized_end=2729
  _globals['_SCHEDULESTARTUPRESPONSE']._serialized_start=2731
  _globals['_SCHEDULESTARTUPRESPONSE']._serialized_end=2851
  _globals['_SCHEDULESHUTDOWNREQUEST']._serialized_start=2853
  _globals['_SCHEDULESHUTDOWNREQUEST']._serialized_end=2916
  _globals['_SCHEDULESHUTDOWNRESPONSE']._serialized_start=2918
  _globals['_SCHEDULESHUTDOWNRESPONSE']._serialized_end=3039
  _globals['_REBOOTREQUEST']._serialized_start=3041
  _globals['_REBOOTREQUEST']._serialized_end=3056
  _globals['_REBOOTRESPONSE']._serialized_start=3058
  _globals['_REBOOTRESPONSE']._serialized_end=3131
  _globals['_SHUTDOWNREQUEST']._serialized_start=3133
  _globals['_SHUTDOWNREQUEST']._serialized_end=3150
  _globals['_SHUTDOWNRESPONSE']._serialized_start=3152
  _globals['_SHUTDOWNRESPONSE']._serialized_end=3227
  _globals['_GETTEMPERATUREREQUEST']._serialized_start=3229
  _globals['_GETTEMPERATUREREQUEST']._serialized_end=3252
  _globals['_GETTEMPERATURERESPONSE']._serialized_start=3254
  _globals['_GETTEMPERATURERESPONSE']._serialized_end=3377
  _globals['_GETINPUTVOLTAGEREQUEST']._serialized_start=3379
  _globals['_GETINPUTVOLTAGEREQUEST']._serialized_end=3403
  _globals['_GETINPUTVOLTAGERESPONSE']._serialized_start=3405
  _globals['_GETINPUTVOLTAGERESPONSE']._serialized_end=3521
  _globals['_LOADFIRMWAREREQUEST']._serialized_start=3523
  _globals['_LOADFIRMWAREREQUEST']._serialized_end=3573
  _globals['_LOADFIRMWARERESPONSE']._serialized_start=3575
  _globals['_LOADFIRMWARERESPONSE']._serialized_end=3654
  _globals['_LOADBOOTLOADERREQUEST']._serialized_start=3656
  _globals['_LOADBOOTLOADERREQUEST']._serialized_end=3692
  _globals['_LOADBOOTLOADERRESPONSE']._serialized_start=3694
  _globals['_LOADBOOTLOADERRESPONSE']._serialized_end=3775
  _globals['_GETFIRMWAREVERSIONREQUEST']._serialized_start=3777
  _globals['_GETFIRMWAREVERSIONREQUEST']._serialized_end=3804
  _globals['_GETFIRMWAREVERSIONRESPONSE']._serialized_start=3806
  _globals['_GETFIRMWAREVERSIONRESPONSE']._serialized_end=3908
  _globals['_GETLOCATIONREQUEST']._serialized_start=3910
  _globals['_GETLOCATIONREQUEST']._serialized_end=3930
  _globals['_GETLOCATIONRESPONSE']._serialized_start=3933
  _globals['_GETLOCATIONRESPONSE']._serialized_end=4321
  _globals['_SYNCRTCTIMEREQUEST']._serialized_start=4323
  _globals['_SYNCRTCTIMEREQUEST']._serialized_end=4343
  _globals['_SYNCRTCTIMERESPONSE']._serialized_start=4345
  _globals['_SYNCRTCTIMERESPONSE']._serialized_end=4423
  _globals['_EVENTDETAIL']._serialized_start=4426
  _globals['_EVENTDETAIL']._serialized_end=4554
  _globals['_GETEVENTSREQUEST']._serialized_start=4556
  _globals['_GETEVENTSREQUEST']._serialized_end=4616
  _globals['_GETEVENTSRESPONSE']._serialized_start=4619
  _globals['_GETEVENTSRESPONSE']._serialized_end=4786
  _globals['_GETDIEVENTSREQUEST']._serialized_start=4789
  _globals['_GETDIEVENTSREQUEST']._serialized_end=4928
  _globals['_GETDIEVENTSRESPONSE']._serialized_start=4931
  _globals['_GETDIEVENTSRESPONSE']._serialized_end=5100
  _globals['_PLATFORMIFACE']._serialized_start=5103
  _globals['_PLATFORMIFACE']._serialized_end=7594
# @@protoc_insertion_point(module_scope)



from contextlib import suppress

from .channel import Channel


class Agent:
    """Represents an agent in the Doover site.

    Attributes
    ----------
    key: str
        The unique identifier for the agent.
    type: str
        The type of the agent, e.g., "doover_users | user".
    name: str
        The name of the agent, if available.
    owner_org: str
        The organization that the agent belongs to.
    deployment_config: dict
        The deployment configuration for the agent, if available.
    channels: list[Channel]
        A list of channels associated with the agent.
    """

    def __init__(self, client, data):
        self.client = client

        self.id = None
        self.key = None
        self.type = None
        self.name = None
        self.owner_org = None
        self.deployment_config = None
        self.channels = None

        self._from_data(data)

    def _from_data(self, data):
        # {'agent': '9fb5d629-ce7f-4b08-b17a-c267cbcd0427', 'current_time': 1715662485.485132, 'type': 'doover_users | user', 'channels': [{'channel': 'd1c7e8e3-f47b-4c68-86d7-65054d9e97d3', 'name': 'josh-test', 'type': 'base', 'agent': '9fb5d629-ce7f-4b08-b17a-c267cbcd0427'}, {'channel': '1f71b8bd-9444-4f34-859f-f339875a765c', 'name': 'test-logs', 'type': 'base', 'agent': '9fb5d629-ce7f-4b08-b17a-c267cbcd0427'}, {'channel': '86c96181-425d-46f4-a4eb-b36bde2b3984', 'name': 'notifications', 'type': 'base', 'agent': '9fb5d629-ce7f-4b08-b17a-c267cbcd0427'}]}
        self.id = data["agent"]
        self.key = data["agent"]

        self.type = data["type"]

        with suppress(KeyError):
            self.name = data["name"]

        with suppress(KeyError):
            self.owner_org = data["owner_org"]

        with suppress(KeyError):
            self.deployment_config = data["settings"]["deployment_config"]

        self.channels = [
            Channel(data=c, client=self.client) for c in data.get("channels", [])
        ]

    @property
    def agent_id(self):
        """Returns the unique identifier for the agent."""
        return self.id

    def update(self):
        """Fetches the latest data for the agent from the API and updates the instance attributes."""
        res = self.client._get_agent_raw(self.id)
        return self._from_data(res)



# fixme: make this an optional package
try:
    from transitions.extensions.asyncio import AsyncTimeout, AsyncMachine
    from transitions.extensions.states import add_state_features
except ImportError:
    print(
        "Transitions module not found. State machine functionality will not be available."
    )
    StateMachine = None
else:

    @add_state_features(AsyncTimeout)
    class StateMachine(AsyncMachine):
        """Async state machine with timeout support."""



import copy
import json
from typing import Any


def maybe_load_json(data):
    """Attempt to load a JSON string, return the original data if it fails."""
    try:
        return json.loads(data)
    except json.JSONDecodeError:
        return data


def apply_diff(
    data: dict[str, Any],
    diff: dict[str, Any],
    do_delete: bool = True,
    clone: bool = True,
):
    """Apply a doover compatible diff to a JSON / dict object.

    Returns a new object with the diff applied.

    To modify the object in-place, pass `clone=False`.
    """
    if clone:
        data = copy.deepcopy(data)

    if not isinstance(diff, dict) or not isinstance(data, dict):
        # if data is not a dict, we can't apply the diff
        # so we just return the diff
        return diff

    for k, v in diff.items():
        if isinstance(v, dict):
            # print(f"applying: ({type(k)}) {k} -> ({type(v)}) {v} (data: ({type(data)}) {data})")
            data[k] = apply_diff(data.get(k, {}), v, do_delete=do_delete, clone=False)
        elif v is None:
            # del data[k]
            if do_delete:
                # if do_delete is True, remove the key from the dict
                data.pop(k, None)
            else:
                # if do_delete is False, set the key to None
                data[k] = None
        else:
            data[k] = v
    return data


def generate_diff(old, new):
    """Generate a doover compatible diff between two JSON / dict objects.

    A diff will contain all keys that are different between the two objects.

    Any keys that are in the old object but not in the new object will be set to None.
    """
    if not isinstance(new, dict) or not isinstance(old, dict):
        return new

    diff = {}
    for k, v in new.items():
        if isinstance(v, dict):
            d = generate_diff(old.get(k, {}), v)
            if d:
                diff[k] = generate_diff(old.get(k, {}), v)
        elif k not in old or old[k] != v:
            diff[k] = v
    for k in old.keys() - new.keys():
        diff[k] = None
    return diff



import logging
import sys
import time

from typing import Any

from ..api import Client, Channel, Message
from ...ui import UIManager

# use the root logger since we want to pipe these logs to a channel.
log = logging.getLogger()


class LogHandler(logging.NullHandler):
    def __init__(self, *args, **kwargs):
        self.logs = []
        super().__init__(*args, **kwargs)

    def handle(self, record):
        if record.levelno < self.level:
            return

        fmt = self.format(record)
        self.logs.append(fmt)

    def emit(self, record):
        self.handle(record)

    def get_logs(self):
        return "\n".join(self.logs)


class ProcessorBase:
    """Base class for all Doover processors.

    Processors can be thought of as tiny lambda functions, and are run in response to messages processed through a channel.
    This base class is designed to be overridden by a user defined class

    In the doover_config.json file we have defined some of these subscriptions
    These are under 'processor_deployments' > 'tasks'

    Examples
    --------

    A simple processor that logs a message to a channel::

        from pydoover.cloud.processor import ProcessorBase

        class target(ProcessorBase):
            def setup(self):
                pass
            def process(self):
                logging.info("Processing message...")


    Attributes
    ----------
    agent_id  : str
        The Doover agent invoking the task
    app_key   : str
        The application key for the processor. This is currently blank.
    access_token : str
        A temporary token that can be used to interact with the Doover API
    log_channel_id : str
        A channel to publish any logs to. This is handled for you through the `logging` module.
    task_id : str
        The identifier string of the task channel which invoked this processor.
    api : :class:`pydoover.cloud.api.Client`
        The Doover API client used to interact with the Doover API.
    ui_manager : :class:`pydoover.ui.UIManager`
        A UI manager to manage the UI for this processor. This is optional and can be used to push updates to the UI.
    message: :class:`pydoover.cloud.api.Message`
        The message that triggered this processor. This is may be `None` depending on the trigger.
    deployment_config : dict[str, Any]
        A dictionary of the deployment configuration for this agent.
    package_config : dict[str, Any]
        A dictionary of the package configuration for this processor.
    """

    def __init__(self, **kwargs):
        self.app_key: str = kwargs.get("app_key", "app")
        self.access_token: str = kwargs["access_token"]

        self._log_handler = LogHandler()
        log.addHandler(self._log_handler)
        log.setLevel(level=logging.INFO)

        self.agent_id: str = kwargs["agent_id"]
        self.log_channel_id: str = kwargs["log_channel"]
        self.task_id: str = kwargs["task_id"]

        self.api: Client = Client(
            token=self.access_token, base_url=kwargs["api_endpoint"]
        )
        self.ui_manager: UIManager = UIManager(self.agent_id, self.app_key, self.api)

        self.deployment_config: dict[str, Any] = kwargs["agent_settings"].get(
            "deployment_config", {}
        )
        self.package_config: dict[str, Any] = kwargs.get("package_config", {})

        try:
            if kwargs["msg_obj"] is None:
                raise KeyError
            self.message = Message(
                client=self.api, data=kwargs["msg_obj"], channel_id=None
            )
        except KeyError:
            self.message = None

        ### kwarg
        #     'agent_id' : The Doover agent id invoking the task e.g. '9843b273-6580-4520-bdb0-0afb7bfec049'
        #     'access_token' : A temporary token that can be used to interact with the Doover API .e.g 'ABCDEFGHJKLMNOPQRSTUVWXYZ123456890',
        #     'api_endpoint' : The API endpoint to interact with e.g. "https://my.doover.com",
        #     'package_config' : A dictionary object with configuration for the task - as stored in the task channel in Doover,
        #     'msg_obj' : A dictionary object of the msg that has invoked this task,
        #     'task_id' : The identifier string of the task channel used to run this processor,
        #     'log_channel' : The identifier string of the channel to publish any logs to
        #     'agent_settings' : {
        #       'deployment_config' : {} # a dictionary of the deployment config for this agent
        #     }

    def setup(self):
        """The setup function to be invoked before any processing of a message.

        This is designed to be overridden by a user to perform any setup required.

        You do **not** need to call `super().setup()` as this function ordinarily does nothing.
        """
        return NotImplemented

    def process(self):
        """Override this method to perform the actual processing of a message.

        This method is invoked after the setup method, and is where the main logic of the processor should be implemented.

        You do **not** need to call `super().process()` as this function ordinarily does nothing.

        You can optionally update the UI at the end of this method by calling `self.ui_manager.push()`.
        """
        return NotImplemented

    def close(self):
        """Override this method to change behaviour before a processor exits.

        This is invoked after the processing of a message is complete, and can be used to clean up resources or perform any final actions.

        You do **not** need to call `super().close()` as this function ordinarily does nothing.
        """
        return NotImplemented

    def execute(self):
        start_time = time.time()
        log.info(f"Initialising processor task for task channel {self.task_id}")
        log.info(f"Started at {start_time}.")

        try:
            self.import_modules()
            self.setup()

            try:
                self.process()
            except Exception as e:
                log.error(f"ERROR attempting to process message: {e} ", exc_info=e)

        except Exception as e:
            log.error(f"ERROR attempting to initialise process: {e}", exc_info=e)

        try:
            self.close()
        except Exception as e:
            log.error(f"ERROR attempting to close process: {e} ", exc_info=e)

        end_time = time.time()
        log.info(
            f"Finished at {end_time}. Process took {end_time - start_time} seconds."
        )

        if self._log_handler.get_logs() and self.log_channel_id is not None:
            self.api.publish_to_channel(
                self.log_channel_id, self._log_handler.get_logs()
            )

    @staticmethod
    def import_modules():
        """Remove all dangling instances of pydoover modules from the python path."""
        if "pydoover" in sys.modules:
            del sys.modules["pydoover"]
        try:
            del pydoover  # noqa: F821
        except NameError:
            pass
        try:
            del pd  # noqa: F821
        except NameError:
            pass

    def get_agent_config(self, filter_key: str = None) -> Any | None:
        """Fetch a config entry for the agent who owns (or invoked) this processor.

        Parameters
        ----------
        filter_key : str, optional
            If provided, will return only the value for this key in the config. Otherwise, returns the entire config dictionary.

        Returns
        -------
        dict or None
            The agent's deployment configuration dictionary, or None if no deployment config is set, or the key is not found.
        """
        if not self.deployment_config:
            return None
        if filter_key:
            return self.deployment_config.get(filter_key)
        return self.deployment_config

    def fetch_channel(self, channel_key: str) -> Channel:
        """Helper method to fetch a channel by its key.

        Parameters
        ----------
        channel_key : str
            The key of the channel to fetch.

        Returns
        -------
        :class:`pydoover.cloud.api.Channel`
            The channel object corresponding to the provided key.

        Raises
        -------
        :class:`pydoover.cloud.api.NotFound`
            If the channel with the specified key does not exist.
        """
        return self.api.get_channel(channel_key)

    def fetch_channel_named(self, channel_name: str) -> Channel:
        """Helper method to fetch a channel by its name, owned by the current agent.

        Parameters
        ----------
        channel_name : str
            The name of the channel to fetch.

        Returns
        -------
        :class:`pydoover.cloud.api.Channel`
            The channel object corresponding to the provided key.

        Raises
        -------
        :class:`pydoover.cloud.api.NotFound`
            If the channel with the specified key does not exist.
        """

        return self.api.get_channel_named(channel_name, self.agent_id)



from .application import Application as Application, run_app as run_app
from .device_agent import DeviceAgentInterface as DeviceAgentInterface
from .modbus import (
    ModbusInterface as ModbusInterface,
    ModbusConfig as ModbusConfig,
    ManyModbusConfig as ManyModbusConfig,
)
from .platform import (
    PlatformInterface as PlatformInterface,
    PulseCounter as PulseCounter,
)



from typing import Union, Any


class NotSet:
    """A sentinel value to indicate that a value has not been set."""


class Colour:
    """A class representing a colour, which can be used in various UI elements.

    This class provides a set of predefined colour constants that can be used directly,
    or allows conversion from hex strings and arbitrary strings to a colour representation.

    In reality, the Doover Site will accept any HTML colour name or hex string, but this class allows
    for future implementations of colour objects if needed.

    Attributes
    ----------
    blue
    yellow
    red
    green
    magenta
    limegreen
    tomato
    orange
    purple
    grey
    """

    blue = "blue"
    yellow = "yellow"
    red = "red"
    green = "green"
    magenta = "magenta"
    limegreen = "limegreen"
    tomato = "tomato"
    orange = "orange"
    purple = "purple"
    grey = "grey"

    @classmethod
    def from_hex(cls, hex_string):
        """Convert a hex string to a colour string.

        This method takes a hex string (e.g., "#FF5733") that can be used where a Colour object is accepted.
        """
        return hex_string

    @classmethod
    def from_string(cls, value):
        """Convert an arbitrary string to a Colour object.

        This method takes an HTML colour name (e.g. `brown`) and allows it to be used where a Colour object is accepted.
        """
        return value  # fixme: this hackiness


class Range:
    """Represents a range of values with associated properties for UI display.

    Attributes
    ----------
    label: str
        A label for the range, used for display purposes.
    min: Union[int, float]
        The minimum value of the range.
    max: Union[int, float]
        The maximum value of the range.
    colour: Colour
        The colour associated with the range, used for display.
    show_on_graph: bool
        Whether to show this range on the graph.
    """

    def __init__(
        self,
        label: str = None,
        min_val: Union[int, float] = None,
        max_val: Union[int, float] = None,
        colour: "Colour" = Colour.blue,
        show_on_graph: bool = True,
    ):
        self.label = label
        self.min = min_val
        self.max = max_val
        self.colour = colour
        self.show_on_graph = show_on_graph

    def to_dict(self):
        to_return = {
            "min": self.min,
            "max": self.max,
            "colour": self.colour,
            "show_on_graph": self.show_on_graph,
        }
        if self.label:
            to_return["label"] = self.label
        return to_return

    @classmethod
    def from_dict(cls, data: dict[str, Any]):
        return cls(
            data.get("label"),
            data["min"],
            data["max"],
            Colour.from_string(data["colour"]),
            data["show_on_graph"],
        )


class Option:
    """Represents an option for a UI element, such as a dropdown or selection list.

    Attributes
    ----------
    name: str
        The name of the option, used for identification.
    display_name: str
        The display name of the option, used for user interface representation.
    """

    def __init__(self, name: str, display_name: str):
        self.name = name
        self.display_name = display_name

    def to_dict(self):
        return {
            "name": self.name,
            "displayString": self.display_name,
            "type": "uiElement",
        }

    @classmethod
    def from_dict(cls, data: dict[str, Any]):
        return cls(data["name"], data["display_str"])


class Widget:
    """Base class for UI widgets, which can be used to represent various UI elements.

    There are two main types of widgets: linear and radial gauges.

    Attributes
    ----------
    linear
        Represents a linear gauge widget.
    radial
        Represents a radial gauge widget.
    """

    linear = "linearGauge"
    radial = "radialGauge"

    @classmethod
    def from_string(cls, value):
        """Convert an arbitrary string to a Widget object.

        This method takes a string (e.g. `linearGauge`) and allows it to be used where a Widget object is accepted.

        This allows for future implementations of Widget objects if needed.
        """
        return value  # fixme: this hackiness



import time


class PID:
    def __init__(self, Kp, Ki, Kd, setpoint=0, output_limits=(None, None)):
        """
        Initialize the PID controller.

        :param Kp: Proportional gain
        :param Ki: Integral gain
        :param Kd: Derivative gain
        :param setpoint: The target value that the PID controller tries to achieve
        :param output_limits: Tuple (min_output, max_output) for limiting output
        """
        self.Kp = Kp
        self.Ki = Ki
        self.Kd = Kd
        self.setpoint = setpoint
        self.output_limits = output_limits

        self._last_time = None
        self._last_error = None
        self._integral = 0
        self._last_output = 0

    def update(self, feedback_value, dt=None):
        """
        Update the PID loop with the current feedback value.

        :param feedback_value: The current value from the process
        :param dt: Optional time interval. If not provided, it's calculated internally.
        :return: The control output
        """
        current_time = time.time()
        error = self.setpoint - feedback_value

        if self._last_time is None:
            # First call, just initialize and return 0
            self._last_time = current_time
            self._last_error = error
            if self._last_output is not None:
                return self._last_output
            return 0

        # Calculate time difference (dt) if not provided
        if dt is None:
            delta_time = current_time - self._last_time
        else:
            delta_time = dt

        # Ensure we don't update too frequently
        if delta_time <= 0.0:
            return self._last_output

        # Proportional term
        proportional = self.Kp * error

        # Integral term
        self._integral += error * delta_time
        integral = self.Ki * self._integral

        # Derivative term
        delta_error = error - self._last_error
        derivative = self.Kd * (delta_error / delta_time)

        # Compute the output
        output = proportional + integral + derivative

        # Limit the output to specified limits
        min_output, max_output = self.output_limits
        if min_output is not None:
            output = max(min_output, output)
        if max_output is not None:
            output = min(max_output, output)

        # Store values for the next loop iteration
        self._last_output = output
        self._last_time = current_time
        self._last_error = error

        return output

    def set_output_limits(self, min_output, max_output):
        """
        Set the minimum and maximum output limits.

        :param min_output: Minimum limit
        :param max_output: Maximum limit
        """
        self.output_limits = (min_output, max_output)

    def set_setpoint(self, setpoint):
        """
        Set a new target value for the PID to reach.

        :param setpoint: The target value
        """
        self.setpoint = setpoint

    def set_last_output(self, output):
        """
        Set the last output value.

        :param output: The last output value
        """
        self._last_output = output

    def set_last_error(self, error):
        """
        Set the last error value.

        :param error: The last error value
        """
        self._last_error = error

    def set_integral_output(self, integral_output):
        """
        Initialise the integral output for a desired output value.

        :param integral: The integral integral_output value
        """
        if self.Ki == 0:
            self._integral = 0
        else:
            self._integral = integral_output / self.Ki

    def reset(self):
        """
        Reset the internal state of the PID controller.
        """
        self._last_time = None
        self._last_error = None
        self._integral = 0
        self._last_output = 0
